package de.ozgcloud.nachrichten.postfach.osiv2;

import java.util.stream.Stream;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.stereotype.Service;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.PostfachRemoteService;
import de.ozgcloud.nachrichten.postfach.osiv2.config.Osi2PostfachProperties;
import de.ozgcloud.nachrichten.postfach.osiv2.transfer.PostfachApiFacadeService;
import lombok.RequiredArgsConstructor;
import lombok.extern.log4j.Log4j2;

@Service
@ConditionalOnProperty(prefix = Osi2PostfachProperties.PREFIX, name = "enabled", havingValue = "true")
@Log4j2
@RequiredArgsConstructor
public class OsiPostfachRemoteService implements PostfachRemoteService {
	private final PostfachApiFacadeService postfachApiFacadeService;

	public static final String POSTFACH_TYPE_OSI = "OSI";

	@Override
	public void sendMessage(PostfachNachricht nachricht) {
		try {
			postfachApiFacadeService.sendMessage(nachricht);
		} catch (RuntimeException e) {
			throw new OsiPostfachException("Failed to send message", e);
		}
	}

	@Override
	public Stream<PostfachNachricht> getAllMessages() {
		try {
			return postfachApiFacadeService.receiveMessages();
		} catch (RuntimeException e) {
			throw new OsiPostfachException("Failed to get messages", e);
		}
	}

	@Override
	public void deleteMessage(String messageId) {
		try {
			postfachApiFacadeService.deleteMessage(messageId);
		} catch (RuntimeException e) {
			throw new OsiPostfachException("Failed to delete messages", e);
		}
	}

	@Override
	public String getPostfachType() {
		return POSTFACH_TYPE_OSI;
	}

	@Override
	public boolean isReplyAllowed() {
		return true;
	}
}
