package de.ozgcloud.nachrichten.postfach.osiv2.config;

import jakarta.annotation.Nullable;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Configuration;

import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.Setter;

@Getter
@Setter
@Configuration
@ConditionalOnProperty(prefix = Osi2PostfachProperties.PREFIX, name = "enabled", havingValue = "true")
@RequiredArgsConstructor
public class Osi2PostfachProperties {

	public static final String PREFIX = "ozgcloud.osiv2";

	private boolean enabled;

	private final ApiConfiguration api;

	private final ProxyConfiguration httpProxy;

	@Getter
	@Setter
	@Configuration
	@ConfigurationProperties(prefix = ApiConfiguration.PREFIX)
	public static class ApiConfiguration {
		public static final String PREFIX = Osi2PostfachProperties.PREFIX + ".api";

		private String resource;
		private String url;
		private String tenant;
		private String nameIdentifier;
	}

	/**
	 * HTTP proxy configuration. To use basic HTTP authentication configure username and password. (See <a
	 * href="https://github.com/reactor/reactor-netty/blob/3b74fcf2f5b2bdd129636e9c55608f0187ef7bd4/reactor-netty-core/src/main/java/reactor/netty/transport/ProxyProvider.java#L142">ProxyProvider</a>)
	 */
	@Getter
	@Setter
	@Configuration
	@ConfigurationProperties(prefix = ProxyConfiguration.PREFIX)
	public static class ProxyConfiguration {
		public static final String PREFIX = Osi2PostfachProperties.PREFIX + ".proxy";

		private boolean enabled;

		private String host;
		private Integer port;

		@Nullable
		private String username = null;
		@Nullable
		private String password = null;
	}
}
