package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import static org.assertj.core.api.Assertions.*;

import java.time.ZonedDateTime;
import java.time.temporal.ChronoUnit;
import java.util.UUID;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mapstruct.factory.Mappers;
import org.mockito.InjectMocks;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.V1ReplyMessageTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyMessage;

class Osi2ResponseMapperTest {

	@InjectMocks
	private Osi2ResponseMapper mapper = Mappers.getMapper(Osi2ResponseMapper.class);
	private V1ReplyMessage message = V1ReplyMessageTestFactory.create();

	@DisplayName("map V1ReplyMessage to PostfachNachricht")
	@Nested
	class V1ReplyMessageToPostfachNachricht {
		@Test
		void shouldHaveId(){
			var postfachNachricht = mapper.toPostfachNachricht(message);

			assertThat(postfachNachricht.getId()).isEqualTo(UUID.nameUUIDFromBytes("123-guid-456".getBytes()).toString());
		}

		@Test
		void shouldHaveVorgangId(){
			var postfachNachricht = mapper.toPostfachNachricht(message);

			assertThat(postfachNachricht.getVorgangId()).isEqualTo("OZG-Cloud-VorgangId");
		}

		@Test
		void shouldHavePostfachAddress(){
			var postfachNachricht = mapper.toPostfachNachricht(message);

			assertThat(postfachNachricht.getPostfachAddress().getIdentifier().toString())
					.hasToString(UUID.nameUUIDFromBytes("Mailbox-Id-Antwortender".getBytes()).toString());
		}

		@Test
		void shouldHaveCreatedAt(){
			var postfachNachricht = mapper.toPostfachNachricht(message);

			assertThat(postfachNachricht.getCreatedAt()).isNotNull().isCloseTo(ZonedDateTime.now(), within(5, ChronoUnit.SECONDS));
		}

		@Test
		void shouldHaveCreatedBy(){
			var postfachNachricht = mapper.toPostfachNachricht(message);

			assertThat(postfachNachricht.getCreatedBy()).isEqualTo("Das ist der Absender");
		}

		@Test
		void shouldHaveSentAt(){
			var postfachNachricht = mapper.toPostfachNachricht(message);

			assertThat(postfachNachricht.getSentAt()).isNotNull().isCloseTo(ZonedDateTime.now(), within(5, ChronoUnit.SECONDS));
		}

		@Test
		void shouldHaveDirection(){
			var postfachNachricht = mapper.toPostfachNachricht(message);

			assertThat(postfachNachricht.getDirection()).isEqualTo(PostfachNachricht.Direction.IN);
		}

		@Test
		void shouldHaveSubject(){
			var postfachNachricht = mapper.toPostfachNachricht(message);

			assertThat(postfachNachricht.getSubject()).isEqualTo("Das ist das Subject");
		}

		@Test
		void shouldHaveBody(){
			var postfachNachricht = mapper.toPostfachNachricht(message);

			assertThat(postfachNachricht.getMailBody()).isEqualTo("""
			Das ist das Multiline
			Body""");
		}

		@Test
		void shouldHaveReplyOption(){
			var postfachNachricht = mapper.toPostfachNachricht(message);

			assertThat(postfachNachricht.getReplyOption()).isEqualTo(PostfachNachricht.ReplyOption.POSSIBLE);
		}

//		TODO:prüfen das Attachments in der PostfachNachricht enthalten sind
	}
}
