package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import static de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachAddressTestFactory.*;
import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;

import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachNachrichtTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.api.MessageExchangeApi;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MessageExchangeDeleteMessageResponse;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MessageExchangeReceiveMessage;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MessageExchangeReceiveMessagesResponse;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.MessageExchangeSendMessageResponse;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.OutSendMessageRequestV2;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyMessage;

class PostfachApiFacadeServiceTest {

	@InjectMocks
	PostfachApiFacadeService postfachApiFacadeService;

	@Mock
	MessageExchangeApi messageExchangeApi;

	@Mock
	Osi2RequestMapper osi2RequestMapper;

	@Mock
	Osi2ResponseMapper osi2ResponseMapper;

	@DisplayName("send message")
	@Nested
	class TestSendMessage {

		@Mock
		OutSendMessageRequestV2 outSendMessageRequestV2;

		@Mock
		MessageExchangeSendMessageResponse messageExchangeSendMessageResponse;

		private final PostfachNachricht nachricht = PostfachNachrichtTestFactory.create();

		@BeforeEach
		void mock() {
			when(osi2RequestMapper.mapMailboxId(nachricht)).thenReturn(MAILBOX_ID);
			when(osi2RequestMapper.mapOutSendMessageRequestV2(nachricht)).thenReturn(outSendMessageRequestV2);
			when(messageExchangeApi.sendMessage(any(), any())).thenReturn(messageExchangeSendMessageResponse);
		}

		@DisplayName("should call sendMessage")
		@Test
		void shouldCallSendMessage() {
			postfachApiFacadeService.sendMessage(nachricht);

			verify(messageExchangeApi).sendMessage(MAILBOX_ID, outSendMessageRequestV2);

		}
	}

	@DisplayName("receive messages")
	@Nested
	class TestReceiveMessage{

		@Mock
		MessageExchangeReceiveMessagesResponse messageExchangeReceiveMessagesResponse;

		@BeforeEach
		void mock(){
			when(messageExchangeApi.receiveMessages(anyInt(),anyInt())).thenReturn(messageExchangeReceiveMessagesResponse);
		}

		@Test
		void responseShouldNotBeEmpty(){
			postfachApiFacadeService.receiveMessages();

			verify(messageExchangeApi).receiveMessages(anyInt(), anyInt());
		}
	}

	@DisplayName("fetch Message by guid")
	@Nested
	class TestFetchMessageByGuid{

		@Mock
		V1ReplyMessage replyMessage;
		@Mock
		MessageExchangeReceiveMessage receiveMessage;

		@Test
		void shouldCallGetMessage(){
			when(messageExchangeApi.getMessage(any())).thenReturn(replyMessage);

			postfachApiFacadeService.fetchMessageByGuid(receiveMessage);

			verify(messageExchangeApi).getMessage(any());
		}

		@Test
		void shouldCallResponseMapper(){
			when(messageExchangeApi.getMessage(any())).thenReturn(replyMessage);
			when(osi2ResponseMapper.toPostfachNachricht(any())).thenReturn(PostfachNachrichtTestFactory.create());

			postfachApiFacadeService.fetchMessageByGuid(receiveMessage);

			verify(osi2ResponseMapper).toPostfachNachricht(any());
		}

		@Test
		void shouldReturnPostfachNachricht(){
			when(messageExchangeApi.getMessage(any())).thenReturn(replyMessage);
			when(osi2ResponseMapper.toPostfachNachricht(any())).thenReturn(PostfachNachrichtTestFactory.create());

			var postfachNachricht = postfachApiFacadeService.fetchMessageByGuid(receiveMessage);

			assertThat(postfachNachricht).isInstanceOf(PostfachNachricht.class);
		}
	}

	@DisplayName("Delete Message")
	@Nested
	class TestDeleteMessage{
		@Mock
		MessageExchangeDeleteMessageResponse replyMessage;

		@Test
		void shouldCallDeleteMessage(){
			when(messageExchangeApi.deleteMessage(any())).thenReturn(replyMessage);

			postfachApiFacadeService.deleteMessage(UUID.randomUUID().toString());

			verify(messageExchangeApi).deleteMessage(any());
		}
	}

}